==> org-lsst-ccs-groovy-description-4.0.9-sources.jar: org/lsst/ccs/description/ComponentNode.java <==
package org.lsst.ccs.description;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.*;
import org.lsst.ccs.utilities.logging.Logger;


/**
 * Abstract class for a node in the concrete implementation of a subsystem description.
 * It is a standard tree node structure containing in addition a key (the name
 * of the component) and the actual component that will be part of the modular 
 * structure of the subsystem.
 * Subclasses define how the component is built, by overriding {@code setComponent()}
 * 
 * @param <T> The particular instance of the ComponentNode.
 *
 * @author The LSST CCS Team
 */
public class ComponentNode <T extends ComponentNode<T>> implements Serializable {
		
    private static final long serialVersionUID = 284121597899337112L;
    
    protected static final Logger logger = Logger.getLogger("org.lsst.ccs.description");
    
    private Map tags = new HashMap();
    
    /** The key of the node. */
    private String key ;

    /** The parent of the node. */
    private ComponentNode<T> parent;
    
    /** The list of children. */
    private final List<T> children = new ArrayList<>();

    /** The component that belongs to the node. It is lazy initialized. */
    private Object component;
    
    private String protocol;
    
    public ComponentNode(T parent, String name) {
        this(parent, name, null);
    }

    public ComponentNode(T parent, String name, Object component) {
        this.parent = parent;
        this.key = name;
        this.component = component;
    }

    public String getProtocol() {
        return protocol;
    }
    public void setProtocol(String protocol) {
        this.protocol = protocol;
    }
    
    public Object getComponent() {
        return component;
    }

    public void setComponent(Object component) {
        if ( this.component != null ) {
            throw new RuntimeException("Cannot set component twice.");
        }
        this.component = component;
        

        // Attempt to set the children component to an existing field of the
        // parent
        for (T child : children) {
            String childKey = child.getKey();
            Field parentField = null;
            Class cls = component.getClass();
            while(cls.getCanonicalName().startsWith("org.lsst")) {
                try {
                    parentField = cls.getDeclaredField(childKey);
                    break;
                } catch (NoSuchFieldException ex) {
                    cls = cls.getSuperclass();
                }
            }
            if (parentField == null) return;
            
            try {
                Object childComp = child.getComponent();
                if (parentField.getType().isAssignableFrom(childComp.getClass())) {
                    parentField.setAccessible(true);
                    parentField.set(component, childComp);
                    logger.warn("field " + parentField.getName() + " of component " + getKey() + " has been automatically set to " + childKey);
                    logger.warn("This feature is to be discontinued and should be replaced with @LookupField annotation");
                } else {
                    logger.warn("could not set field " + childKey + " of " + getKey() + " : classes don't match");
                }
            } catch(IllegalAccessException ex) {
                logger.warn("could not set field " + childKey + " of " + getKey(), ex);
            }
        }
    }
    
    void setParent(ComponentNode<T> parent) {
        this.parent = parent;
    }

    public String getKey() {
        return key;
    }
    
    /**
     * @return the list of child nodes, <TT>null</TT> if there aren't any
     */
    public List<T> getChildren() {
        return Collections.unmodifiableList(children);
    }
    
    /**
     * adds a ComponentNode as a child of the current node
     *
     * @param child to be added to the node
     * @throws NullPointerException if child is null
     */
    public void addChild(T child) throws NullPointerException {
        child.setParent(this);
        this.children.add(child);
    }

    @Override
    public String toString() {
        String valueString ;
        try {
           valueString = String.valueOf(component)  ;
        } catch (Exception exc) {
           valueString = "can't trace value [" + exc +"] "  ;
        }
        return String.valueOf(key) + "("
                + valueString + ") {"
                + String.valueOf(children) + "}";
    }
    
    public ComponentNode<T> getParent() {
        return parent;
    }
    
    protected void setTags(Map tags) {
        this.tags = tags;
    }
    
    public Map getTags() {
        return tags;
    }
}

==> org-lsst-ccs-groovy-description-4.0.10-sources.jar: org/lsst/ccs/description/ComponentNode.java <==
package org.lsst.ccs.description;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.*;
import org.lsst.ccs.utilities.logging.Logger;


/**
 * Abstract class for a node in the concrete implementation of a subsystem description.
 * It is a standard tree node structure containing in addition a key (the name
 * of the component) and the actual component that will be part of the modular 
 * structure of the subsystem.
 * Subclasses define how the component is built, by overriding {@code setComponent()}
 * 
 * @param <T> The particular instance of the ComponentNode.
 *
 * @author The LSST CCS Team
 */
public class ComponentNode <T extends ComponentNode<T>> implements Serializable {
		
    private static final long serialVersionUID = 284121597899337112L;
    
    public static final Logger logger = Logger.getLogger("org.lsst.ccs.description");
    
    protected final Map tags = new HashMap();
        
    /** The key of the node. */
    private String key ;

    /** The parent of the node. */
    private ComponentNode<T> parent;
    
    /** The list of children. */
    private final List<T> children = new ArrayList<>();

    /** The component that belongs to the node. It is lazy initialized. */
    private Object component;
    
    private String protocol;
    
    public ComponentNode(T parent, String name) {
        this(parent, name, null);
    }

    public ComponentNode(T parent, String name, Object component) {
        this.parent = parent;
        this.key = name;
        this.component = component;
    }

    public String getProtocol() {
        return protocol;
    }
    public void setProtocol(String protocol) {
        this.protocol = protocol;
    }
    
    public Object getComponent() {
        return component;
    }

    public void setComponent(Object component) {
        if ( this.component != null ) {
            throw new RuntimeException("Cannot set component twice.");
        }
        this.component = component;
        

        // Attempt to set the children component to an existing field of the
        // parent
        for (T child : children) {
            String childKey = child.getKey();
            Field parentField = null;
            Class cls = component.getClass();
            while(cls.getCanonicalName().startsWith("org.lsst")) {
                try {
                    parentField = cls.getDeclaredField(childKey);
                    break;
                } catch (NoSuchFieldException ex) {
                    cls = cls.getSuperclass();
                }
            }
            if (parentField == null) return;
            
            try {
                Object childComp = child.getComponent();
                if (parentField.getType().isAssignableFrom(childComp.getClass())) {
                    parentField.setAccessible(true);
                    parentField.set(component, childComp);
                    logger.warn("field " + parentField.getName() + " of component " + getKey() + " has been automatically set to " + childKey);
                    logger.warn("This feature is to be discontinued and should be replaced with @LookupField annotation");
                } else {
                    logger.warn("could not set field " + childKey + " of " + getKey() + " : classes don't match");
                }
            } catch(IllegalAccessException ex) {
                logger.warn("could not set field " + childKey + " of " + getKey(), ex);
            }
        }
    }
    
    void setParent(ComponentNode<T> parent) {
        this.parent = parent;
    }

    public String getKey() {
        return key;
    }
    
    /**
     * @return the list of child nodes, <TT>null</TT> if there aren't any
     */
    public List<T> getChildren() {
        return Collections.unmodifiableList(children);
    }
    
    /**
     * adds a ComponentNode as a child of the current node
     *
     * @param child to be added to the node
     * @throws NullPointerException if child is null
     */
    public void addChild(T child) throws NullPointerException {
        child.setParent(this);
        this.children.add(child);
    }

    @Override
    public String toString() {
        String valueString ;
        try {
           valueString = String.valueOf(component)  ;
        } catch (Exception exc) {
           valueString = "can't trace value [" + exc +"] "  ;
        }
        return String.valueOf(key) + "("
                + valueString + ") {"
                + String.valueOf(children) + "}";
    }
    
    public ComponentNode<T> getParent() {
        return parent;
    }
    
    public void addTag(String key, Object value) {
        tags.put(key, value);
    }
    
    public Object getTag(String key) {
        return tags.get(key);
    }
    
    /** Gets the node tags for this node.
     * @return the node tags
     */
    public Map getTags() {
        return tags;
    }
}
